/***********************************************************************/
/*                                                                     */
/*    PROGRAMA SAS PARA ESTIMAR PARMETROS ASSOCIADOS A                */
/*         TABELAS DE VIDA E  FERTILIDADE                              */
/*                                                                     */
/*       NOME: tabela de vida.sas                                      */                                                                 /*                                                                     */
/*      DADOS: exemplo.xls                                             */
/*                                                                     */
/*   PRODUTOS: SAS Language, BASE SAS Procedures, SAS Macro Language   */
/*             and SAS/GRAPH                                           */
/*                                                                     */
/*      PROCS: SORT, MEANS, GPLOT, PRINT                               */
/*                                                                     */
/*       REFS: MAIA, A. de H. N.; LUIZ, A. J. B.; CAMPANHOLA, C..      */
/*                Statistical  Inference on associated lifetable       */
/*                parameters using jackknife technique: computation    */
/*                nal aspects.Journal of  Economic Entomology          */ 
/*                93(2):511-518.                                       */
/***********************************************************************/
/*---------------------------------------------------------------------*/
/*  Leitura do arquivo Excel2000 (*.xls) contendo os dados de          */ 
/*  sobrevivncia e fertilidade                                        */
/*  com as seguintes variveis (colunas)                               */ 
/*  GRUPO - nome ou nmero do "tratamento"                             */
/*  N_FEMEA - no. de identificao de cada fmea                       */  
/*  IDADE - idade da fmea em cada data de oviposio                  */
/*  N_OVOS - nmero de ovos/femea em cada data de oviposio           */
/*  PCT_FEM - porcentagem de fmeas na populao de descendentes       */
/*  SFIMAT - sobrevivncia dos descendentes durante a fase imatura     */
/*---------------------------------------------------------------------*/
PROC IMPORT OUT=DATA1 
     DATAFILE= "C:\nome do diretorio\dados exemplo.xls" /*nome do arquivo*/
     DBMS=EXCEL2000 REPLACE;  /*tipo do arquivo*/
     GETNAMES=YES; /*opo para leitura do nome das variveis na 1. linha */
RUN;
options ls=70 nodate pageno=1; /* largura e numerao da pgina */
run;
proc print data=DATA1;
var GRUPO N_FEMEA IDADE N_OVOS PCT_FEM SFIMAT;
run;
/*---------------------------------------------------------------------*/
/* Construo de grficos do padro de oviposio em cada GRUPO        */
/*---------------------------------------------------------------------*/
/* Ordenao dos dados por grupo e idade*/
data FERT1;
set DATA1;
keep GRUPO N_FEMEA IDADE N_OVOS;
proc sort data=FERT1 out=FERT2;
 by GRUPO IDADE;

proc means data=FERT2 noprint;
        var IDADE N_OVOS;
        output out=FERT3 min=IDADEMIN OVOSMIN max=IDADEMAX OVOSMAX;

/* Definio dos valores mximo e mnimo para as escalas dos eixos x e y */
/* do grfico de oviposio                                              */

data _null_;
        set FERT3;
        IDADEMIN=IDADEMIN-0.5;
        IDADEMAX=IDADEMAX+0.5;
        call symput('IDADEMIN' ,IDADEMIN);
        call symput('IDADEMAX' ,IDADEMAX);
		call symput('OVOSMIN' ,OVOSMIN);
        call symput('OVOSMAX' ,OVOSMAX);

data FERT2;
 set FERT2;
 where N_OVOS>0;

 label IDADE= 'Idade da fmea (dias)';
 label GRUPO= 'Grupo';
 label N_OVOS= 'Nmero de ovos';

/* Formatao do grfico */
goptions reset=global gunit=pct
          ftext='Univers/bold' htext=4.5;
/* title "Padro temporal de oviposio";*/
/* Grupo 1 */
symbol interpol=none    /* tipo de interpolaao     */
        cv=black        /* cor do smbolo           */
        width=1         /* espesura da linha        */
        value=dot       /* tipo de smbolo          */
        height=2;       /* tamanho do smbolo       */
/* Grupo 2 */
symbol2 cv=red value=dot height=2;
/* Grupo 3*/
symbol3 cv=blue value=dot height=2;

/* Formatao do eixo x */
 axis1 label=(angle=0 h=4.5) /* ngulo entre o rtulo e o eixo x
                              e tamanho da fonte */
       offset=(1,1)          /* distancia entre a origem do eixo x
	                          e o primeiro marcador e entre o fim
	                          do eixo x e o ultimo marcador */
       order=&IDADEMIN to &IDADEMAX BY 1    /* escala do eixo x */
       length=90;                          /* Comprimento do eixo x */
       
/* Formatao do eixo y */
 axis2 label=(angle=90 h=4.5)/* ngulo entre o rtulo e o eixo x
                                e tamanho da fonte */
 order=&OVOSMIN to &OVOSMAX BY 10 length=60; /* escala do eixo x */
 run;      

/* Construo do grfico de acordo com a formatao pr-definida */

proc gplot data=FERT2;
       plot N_OVOS*IDADE=GRUPO/         /* comando para construo do grafico */
       haxis=axis1 vaxis=axis2;         /* definio dos eixos  */
run;
/*---------------------------------------------------------------------*/
/* Clculo do nmero total de ovos postos por femea durante o perodo  */
/* de oviposio                                                       */
/*---------------------------------------------------------------------*/
proc sort data=DATA1;
        by GRUPO N_FEMEA IDADE;

proc means data=DATA1 sum noprint;
        bY GRUPO N_FEMEA;
        var N_OVOS;
        output out=FERT4 sum=OVTOTAL;

proc print data=FERT4 noobs;
title 'No.de ovos por fmea durante o perodo total de oviposio';
        by GRUPO;
        var N_FEMEA OVTOTAL;
run;
/*---------------------------------------------------------------------*/
/* Exportao dos dados de numero de  ovos postos por femea durante o  */
/* perodo total de oviposio para um arquivo Excel2000               */
/* (bt vs podisus oviposicao.xls)                                      */
/*---------------------------------------------------------------------*/
proc export data=FERT4
outfile="C:\nome do diretorio\
         oviposicao exemplo.xls"
DBMS=EXCEL2000 replace;
run;
/*---------------------------------------------------------------------------------------*/
/* Construo de box-plots para o no, de ovos  postos por fmea em cada GRUPO            */
/*---------------------------------------------------------------------------------------*/
data FERT4;
 set FERT4;
 label OVTOTAL= 'No.de ovos/fmea';
 label GRUPO= 'GRUPO';

goptions reset=global gunit=pct
ftext='Univers/bold' htext=4.5;

/*title "Box plot do numero de ovos por femea";*/
 symbol interpol=boxtf05  /* tipo de box plot           */
        cv=red            /* cor do interior da "caixa" */
        co=black          /* cor dos limites da "caixa" */
        width=2           /* espessura da linha         */
        value=dot         /* tipo de simbolo            */
        height=2;         /* tamanho do simbolo         */

 axis1 offset=(5,5)length=25;

 axis2 label=(angle=90)length=35;
proc gplot data=FERT4;
    plot OVTOTAL*GRUPO/haxis=axis1 vaxis=axis2;
run;

/*---------------------------------------------------------------------*/
/* Clculo do nmero de indivduos (fmeas) em cada GRUPO              */
/*---------------------------------------------------------------------*/

proc sort data=DATA1 nodupkey out=DATA2;
        by GRUPO N_FEMEA;

proc means noprint data=DATA2;
        var N_OVOS;
        by GRUPO;
        output out=DATA3 n=NINDGR;

proc means noprint data=DATA3;
        var NINDGR;
        output out=DATA4 max=MAXNGR;

proc print data=DATA3 noobs;
title "Nmero de indivduos por grupo";
var GRUPO NINDGR;
run;
/*---------------------------------------------------------------------*/
/* Criao de um arquivo com nmeros de identificao de cada fmea em */
/* ordem seqencial                                                    */
/*---------------------------------------------------------------------*/

data _null_;
        set DATA4;
        call symput('N',MAXNGR);

data T;
        set DATA3;
         do INDNUM=1 to &N;
          if INDNUM>NINDGR then delete;
          drop _type_ _freq_;
         output;
        end;

data T;
merge T DATA2;
        by GRUPO;

data DATA1SEQ;
merge DATA1 T;
        by GRUPO N_FEMEA;

proc sort data=DATA1SEQ;
        by INDNUM;
/*proc print data=DATA1SEQ;*/
run;
/*---------------------------------------------------------------------*/
/* Criao de um arquivo (DATA6)com NG+1 subconjuntos (V), sendo o     */
/* primeiro (V=0) com todas as NG fmeas em cada um dos respectivos    */
/* tratamentos e os outros NG subconjuntos com NG-1 fmeas. Para V=1,  */
/* retira-se a fmea 1 e assim sucessivamente at a fmea NG. Este     */
/* conjunto de dados ser utilizado para estimar os parmetros da TVF  */
/*  utilizando o mtodo jackknife.                                     */
/*---------------------------------------------------------------------*/

data DATA6;
set DATA1SEQ;
        V=0;
        %macro ALI;
        %do I=1 %to &N;
        %let G=&I;

data DATA5;
set DATA1SEQ;
        V=&G;
        if INDNUM=V then delete;

data DATA6;
merge DATA6 DATA5;
        by V;
%end;
%mend ALI;
%ALI
run;
/*---------------------------------------------------------------------*/
/* Clculo do nmero mdio de ovos postos por fmea (NM_OVFEM) em cada */
/*  data de oviposio                                                 */
/*---------------------------------------------------------------------*/

/* Ordenao dos dados */
proc sort data=DATA6;
        by GRUPO V INDNUM IDADE;

data DATA6;
set DATA6;
        if V>NINDGR then delete;

proc sort data=DATA6;
        by GRUPO V IDADE;
/* Cculo do nmero de ovos do sexo feminino, por fmea, em cada data de oviposio */
data DATA6;
set DATA6;
        NM_OVFEM=N_OVOS*PCT_FEM;

proc means data=DATA6 noprint;
        id NINDGR SFIMAT;
        by GRUPO V IDADE;
        var NM_OVFEM;
        output out=DATA7 mean=MX;
run;
/*---------------------------------------------------------------------*/
/*  Construo de tabelas de vida e fertilidade com os dados de cada   */
/*  subconjunto V                                                      */
/*---------------------------------------------------------------------*/

data DATA8;
set DATA7;
        N_FVIVAS=_freq_;
        if V=0 then LX=SFIMAT*(N_FVIVAS/NINDGR);
        if V>0 then LX=SFIMAT*(N_FVIVAS/(NINDGR-1));
        LXMX=LX*MX;
        MXLXX=LXMX*IDADE;
        drop _type_ _freq_;
run;
/*---------------------------------------------------------------------*/
/* Exportao para um arquivo Excel2000, dos dados das tabelas de vida */
/* e fertilidade de cada grupo, construdas com os dados originais(V=0)*/
/*---------------------------------------------------------------------*/
                                 
data DATA8V0;
set DATA8;
where V=0;
keep GRUPO IDADE LX MX LXMX MXLXX;

proc export data=DATA8V0
outfile="C:\nome do diretorio\tabelas de vida exemplo.xls"
DBMS=EXCEL2000 replace;
run;

proc print data=DATA8V0 noobs;
title 'Tabela de vida e fertilidade';
        by GRUPO;
        var IDADE LX MX LXMX MXLXX;
run; 
/*---------------------------------------------------------------------*/
/*  Clculo da taxa lquida de reproduo (Ro, no. fem/fem) e do       */
/*  numerador da expresso IMG=(soma(MX*LX*X))/Ro para estimativa do     */
/*  intervalo entre geraes (IMG) pelo mtodo aproximado.               */
/*---------------------------------------------------------------------*/

proc means noprint data=DATA8;
        id NINDGR;
        by GRUPO V;
        var LXMX MXLXX;
        output out=DATA9 sum=RO NUMT;

/*---------------------------------------------------------------------*/
/* Estimao dos seguintes parmetros utilizando o mtodo aproximado   */
/* Ro: taxa lquida de reproduo                                      */
/* IMG : intervalo mdio entre geraes                                */
/* IMG: tempo de duplicao                                            */
/* Rm: taxa intrnseca de crescimento                                  */
/* Lambda: razo finita de crescimento                                 */
/*---------------------------------------------------------------------*/

data DATA10;
set DATA9;
        IMG=NUMT/RO;
        RM=(LOG(RO))/IMG;
		TD=(log(2))/RM;
		LAMBDA=exp(RM);
        keep GRUPO NINDGR V RO IMG TD RM LAMBDA;
run;

/*---------------------------------------------------------------------*/
/*Impresso na tela "output" das estimativas dos parmetros utilizando */ 
/* mtodo aproximado.                                                  */  
/*---------------------------------------------------------------------*/
proc print data=DATA10 noobs;
title 'Estimativas dos parmetros obtidas pelo o mtodo aproximado';
where V=0;
var GRUPO RO IMG TD RM LAMBDA;
run;
/*--------------------------------------------------------------------*/
/* Estimao dos parmetros utilizando mtodos iterativos             */ 
/*--------------------------------------------------------------------*/
data DATA11;
merge DATA8 DATA10;
        by GRUPO V;
        keep GRUPO NINDGR V IDADE RM RO LXMX;

data DATA12;
set DATA11;
        do U=500 to 1500 by 5;
          R=RM*(U/1000);
          Y=LXMX*(exp(-R*IDADE));
          keep GRUPO NINDGR V IDADE Y U R RO;
         output;
        end;

proc sort data=DATA12;
        by GRUPO V R;

proc means noprint data=DATA12;
        by GRUPO V R;
        var Y;
        id U RO NINDGR;
        output out=DATA13 sum=SUM;
data DATA14;
set DATA13;
        DELTA=abs(1-SUM);
        drop _type_ _freq_;

proc means noprint data=DATA14;
        by GRUPO V;
		var DELTA;
output out=MINDELTA min=MINDELTA;
run;
/*---------------------------------------------------------------------*/
/* Estimao dos parmetros utilizando mtodos iterativos              */ 
/*---------------------------------------------------------------------*/
data DATA15a;
merge DATA14 MINDELTA;
by GRUPO V;

data DATA15;
set DATA15a;
where DELTA=MINDELTA;
        RM=R;
        TD=(log(2))/RM;
        IMG=(log(RO))/RM;
        LAMBDA=exp(RM);
		DIFMETHOD=U/1000;
keep GRUPO NINDGR V SUM MINDELTA RO RM TD IMG LAMBDA;
run;
/*proc print data=data15 noobs;
title "Passos do metodo iterativo";
var GRUPO NINDGR V SUM MINDELTA RO RM TD IMG LAMBDA;
run;*/

data DATA16;
set DATA15;
where V=0;
        RO0=RO;
        RM0=RM;
        IMG0=IMG;
        TD0=TD;
        LAMBDA0=LAMBDA;
        keep GRUPO NINDGR RO0 RM0 IMG0 TD0 LAMBDA0;
run;
proc print data=DATA16 noobs;
title 'Estimativas dos parmetros utilizando o conjunto de dados original';
        by GRUPO;
	  var RO0 RM0 IMG0 TD0 LAMBDA0;
run;
data DATA17;
set DATA15;
        where V>0;
        keep GRUPO NINDGR V RO RM IMG TD LAMBDA;
run;
proc sort data=DATA16;
        by GRUPO;
proc sort data=DATA17;
        by GRUPO;

data DATA18;
merge DATA16 DATA17;
        by GRUPO;
run;
/*---------------------------------------------------------------------*/
/* Clculo dos pseudo-valores para RO(ROPSV), RM(RMPSV), IMG(IMGPSV)   */
/* TD(TDPSV) e LAMBDA(LPSV) utilizando os respectivos valores estima-  */
/* dos para cada subconjunto V e os valores obtidos do conjunto de     */
/* dados original.                                                     */
/*---------------------------------------------------------------------*/
data DATA19;
set DATA18;
        ROPSV=(NINDGR*RO0)-((NINDGR-1)*RO);
        RMPSV=(NINDGR*RM0)-((NINDGR-1)*RM);
        IMGPSV=(NINDGR*IMG0)-((NINDGR-1)*IMG);
        TDPSV=(NINDGR*TD0)-((NINDGR-1)*TD);
        LPSV=(NINDGR*LAMBDA0)-((NINDGR-1)*LAMBDA);
proc print data=DATA19 noobs;
title 'Pseudo-valores de Ro';
        var GRUPO ROPSV;
run;

data PSEUDO;
set DATA19;
keep GRUPO V ROPSV RMPSV TDPSV IMGPSV LPSV;

/*---------------------------------------------------------------------*/
/* Exportao do arquivo de dados (PSEUDO) com os NG pseudo-valores de */
/* Ro, Rm, IMG, Td, e Lambda em cada GRUPO                               */
/*---------------------------------------------------------------------*/

data PSEUDO;
set DATA19;
keep GRUPO V ROPSV RMPSV TDPSV IMGPSV LPSV;
proc export data=PSEUDO
outfile="C:\nome do diretorio\pseudovalores exemplo.xls"
DBMS=EXCEL2000 replace;
run;
/*---------------------------------------------------------------------*/
/* Construo de grficos de caixas para cada parmetro associado     */
/* da tabela de vida em cada GRUPO                                     */  
/*---------------------------------------------------------------------*/
data DATA19;
 set DATA19;
 label ROPSV= 'Ro (fmea/fmea)';
 label RMPSV= 'Rm ';
 label IMGPSV= 'IMG (dias)';
 label TDPSV= 'TD (dias )';
 label LPSV= 'Lambda';
 label GRUPO= 'Grupo';

 goptions reset=global gunit=pct
          ftext='Univers/bold' htext=4.5;

 symbol interpol=boxtf05  /* tipo de box plot           */
        cv=red            /* cor do interior da "caixa" */
        co=black          /* cor dos limites da "caixa" */
        width=2           /* espessura da linha         */
        value=dot         /* tipo de simbolo            */
        height=2;         /* tamanho do simbolo         */

 axis1 offset=(5,5)minor=none length=25;

 axis2 label=(angle=90) minor=none major=(n=5)length=35;

 proc gplot data=DATA19;
 plot (ROPSV RMPSV IMGPSV TDPSV LPSV)*GRUPO/haxis=axis1
                                         vaxis=axis2;
 run;
 quit;
/*---------------------------------------------------------------------*/
/* Criaco de um arquivo de dados (DATA20) com mdias e varincias dos */
/* pseudo-valores de Ro, Rm, IMG, TD, e Lambda.                        */
/*---------------------------------------------------------------------*/
proc means noprint data=DATA19;
       id NINDGR;
       by GRUPO;
       where V>0;
       var ROPSV RMPSV IMGPSV TDPSV LPSV;
       output out=DATA20 mean= RO_MEDIA RM_MEDIA IMG_MEDIA TD_MEDIA L_MEDIA
                           var= RO_VAR RM_VAR IMG_VAR TD_VAR L_VAR;
run;
/*---------------------------------------------------------------------*/
/* Criaco de um arquivo de dados (T_DATA5) com todas as comparaes   */
/* entre pares de grupos. Esses dados sero utilizados para testes t   */
/* considerando varincias no homogeneas                              */
/*---------------------------------------------------------------------*/

data _null_;
set DATA20 nobs=NGR;
call symput('NGR',NGR);

data T_DATA1;
set DATA20;
        GR=_n_;
        GR1=GRUPO;
        GR2=GRUPO;
        GR_A=GR;
        GR_B=GR;

keep GRUPO GR GR_A GR_B GR1 GR2 NINDGR 
     RO_MEDIA RM_MEDIA IMG_MEDIA TD_MEDIA L_MEDIA
     RO_VAR RM_VAR IMG_VAR TD_VAR L_VAR;

data T_DATA1A;
set T_DATA1;
        if GR<&NGR;
        drop GR2 GR_B;

data T_DATA1B;
set T_DATA1;
        if GR>1;
        drop GR1 GR_A;

data T_DATA2;
        F=&NGR-1;
        do GR1=1 to F;
         Z=GR1+1;
          do GR2=Z to &NGR;
          output;
         end;
        end;
data T_DATA3;
merge T_DATA2 T_DATA1A;
        by GR1;
        RO_MEDIA1=RO_MEDIA;
        RM_MEDIA1=RM_MEDIA;
        IMG_MEDIA1=IMG_MEDIA;
        TD_MEDIA1=TD_MEDIA;
        L_MEDIA1=L_MEDIA;
        RO_VAR_A=RO_VAR;
        RM_VAR_A=RM_VAR;
        IMG_VAR_A=IMG_VAR;
        TD_VAR_A=TD_VAR;
        L_VAR_A=L_VAR;
        NINDGR1=NINDGR;
        keep GR1 GR2 GR_A NINDGR1 RO_MEDIA1 RM_MEDIA1 IMG_MEDIA1
         TD_MEDIA1 L_MEDIA1 RO_VAR_A RM_VAR_A IMG_VAR_A TD_VAR_A L_VAR_A;

proc sort data=T_DATA2;
        by GR2 GR1;

proc sort data=T_DATA3;
        by GR2;

data T_DATA4;
merge T_DATA3 T_DATA1B;
      by GR2;
      RO_MEDIA2=RO_MEDIA;
      RO_VAR_B=RO_VAR;
      RM_MEDIA2=RM_MEDIA;
      RM_VAR_B=RM_VAR;
      IMG_MEDIA2=IMG_MEDIA;
      IMG_VAR_B=IMG_VAR;
      TD_MEDIA2=TD_MEDIA;
      TD_VAR_B=TD_VAR;
      L_MEDIA2=L_MEDIA;
      L_VAR_B=L_VAR;
      NINDGR2=NINDGR;

keep GR1 GR2 GR_A GR_B
     NINDGR1 NINDGR2
     RO_MEDIA1 RM_MEDIA1 IMG_MEDIA1 TD_MEDIA1 L_MEDIA1
     RO_VAR_A RM_VAR_A IMG_VAR_A TD_VAR_A L_VAR_A 
     RO_MEDIA2 RM_MEDIA2 IMG_MEDIA2 TD_MEDIA2 L_MEDIA2
     RO_VAR_B RM_VAR_B IMG_VAR_B TD_VAR_B L_VAR_B;

proc sort data=T_DATA4;
        by GR1 GR2;

data T_DATA4;
set T_DATA4;
 N1=NINDGR1;
 N2=NINDGR2;
run;
data T_DATA5;
set T_DATA4;

        PARAM='RO '; MED_A=RO_MEDIA1; MED_B=RO_MEDIA2; VAR_A=RO_VAR_A; VAR_B=RO_VAR_B;
output;

        PARAM='RM '; MED_A=RM_MEDIA1; MED_B=RM_MEDIA2; VAR_A=RM_VAR_A; VAR_B=RM_VAR_B;
output;

        PARAM='IMG'; MED_A=IMG_MEDIA1; MED_B=IMG_MEDIA2; VAR_A=IMG_VAR_A; VAR_B=IMG_VAR_B;
output;

        PARAM='TD '; MED_A=TD_MEDIA1; MED_B=TD_MEDIA2; VAR_A=TD_VAR_A; VAR_B=TD_VAR_B;
output;

        PARAM='L '; MED_A=L_MEDIA1; MED_B=L_MEDIA2; VAR_A=L_VAR_A; VAR_B=L_VAR_B;
        keep GR_A GR_B N1 N2 PARAM MED_A VAR_A MED_B VAR_B;
output;
run;
/*---------------------------------------------------------------------*/
/* Teste t de Student para comparao de pares de tratamentos conside- */
/* rando varincias heterogneas.                                      */
/*---------------------------------------------------------------------*/
data T_TEST;
set T_DATA5;
        EP_A=sqrt(VAR_A/N1);
        EP_B=sqrt(VAR_B/N2);
        DIF=MED_A-MED_B;
        T=DIF/(sqrt((VAR_A/N1)+(VAR_B/N2)));
        GL_NUM= ((VAR_A/N1)+(VAR_B/N2))**2;
        GL_DEN=((VAR_A/N1)**2)/(N1-1)+((VAR_B/N2)**2)/(N2-1);
        GL=GL_NUM/GL_DEN;
        P_BI=2*(1-probt(abs(T),GL));
        P_DIR=1-probt(T,GL);
        P_ESQ=1-probt((-1*T),GL);
proc sort data=T_TEST;
by PARAM;
proc print data=T_TEST noobs;
title 'Test t de Student para comparao entre pares de grupos';
        var PARAM GR_A GR_B MED_A EP_A MED_B EP_B P_BI P_DIR P_ESQ;
run;
